// Generated by CoffeeScript 1.10.0
(function() {
  var buildContent, checkInteger, cleanMetadata, fileCopy, fs, getPath, mergeMetaData, methods, path, saveDocument, saveVersion, slugify, writeDocument;

  fs = require('safefs');

  fileCopy = require('fs-extra').copy;

  path = require('path');

  slugify = function(input) {
    input = input.trim().toLowerCase();
    input = input.replace(/[^a-zA-Z0-9]/g, '-').replace(/^-/, '').replace(/-+/g, '-').trim();
    input = input.replace(/-$/, '').trim();
    return input;
  };

  getPath = function(plugin, title) {
    var defaultSavePath, name;
    defaultSavePath = plugin.getConfig().defaultSavePath;
    name = slugify(title);
    return path.join(defaultSavePath, name + '.html.md');
  };

  writeDocument = function(file, fileContent, docObject, plugin, callback) {
    return fs.writeFile(file, fileContent, function(err) {
      if (err) {
        return callback({
          success: false,
          msg: 'unable to write document'
        });
      } else {
        if (plugin.getConfig().handleRegeneration) {
          plugin.generateUpdate(docObject.docId, file);
        }
        return callback(docObject);
      }
    });
  };

  saveVersion = function(plugin, file, docId, callback) {
    var copyFile, copyPath, dataPath, dt, mill;
    dt = new Date();
    mill = dt.getTime().toString();
    dataPath = plugin.getConfig().dataPath;
    copyPath = path.join(dataPath, 'versions', docId.toString());
    copyFile = mill + ".html.md";
    return fs.mkdir(copyPath, function(err) {
      return fileCopy(file, path.join(copyPath, copyFile), function(err) {
        if (err) {
          callback({
            success: false,
            msg: 'Unable to create version file'
          });
        }
        return callback({
          success: true,
          msg: 'rename successful'
        });
      });
    });
  };

  mergeMetaData = function(plugin, opts) {
    var collectionName, docId, docMeta, docpad, document, key, val;
    docId = opts.docId;
    collectionName = plugin.getConfig().postCollection;
    docpad = plugin.docpad;
    if (docId) {
      document = plugin.findDocument(docId);
      docMeta = document.meta;
      for (key in docMeta) {
        val = docMeta[key];
        if (!opts[key]) {
          opts[key] = val;
        }
      }
    }
    return {
      fullPath: document.fullPath,
      opts: opts
    };
  };

  cleanMetadata = function(docMeta, opts, plugin) {
    var config, customFields, customVal, key, val;
    config = plugin.getConfig();
    customFields = config.customFields;
    for (key in docMeta) {
      val = docMeta[key];
      if (config.allowableMetadata.indexOf(key) === -1) {
        delete docMeta[key];
      }
    }
    for (key in customFields) {
      val = customFields[key];
      if (opts[key]) {
        customVal = opts[key];
        docMeta[key] = customVal.replace(titleReg, '').trim();
      }
    }
    return docMeta;
  };

  buildContent = function(opts, plugin) {
    var config, content, docMeta, fileContent, i, meta, prop, ref, sanitize, titleReg, val;
    docMeta = opts;
    config = plugin.getConfig();
    titleReg = config.titleReg;
    sanitize = config.sanitize;
    content = sanitize(opts.content);
    delete docMeta.content;
    docMeta.title = docMeta.title.replace(titleReg, '').trim();
    if (docMeta.slug) {
      docMeta.slug = slugify(docMeta.slug);
    }
    if (docMeta.tags) {
      if (!Array.isArray(docMeta.tags)) {
        docMeta.tags = docMeta.tags.split(',');
      }
      ref = docMeta.tags;
      for (i in ref) {
        val = ref[i];
        docMeta.tags[i] = val.replace(titleReg, '').trim();
      }
    }
    docMeta.docId = parseInt(docMeta.docId);
    docMeta.layout = docMeta.layout || config.defaultLayout;
    docMeta.editdate = (new Date()).toString();
    docMeta.edit_user = docMeta.user.name;
    docMeta.edit_user_id = docMeta.user.user_id;
    if (!config.allowArbitraryMetadata) {
      docMeta = cleanMetadata(docMeta, opts, plugin);
    }
    meta = "---\n";
    for (prop in docMeta) {
      val = docMeta[prop];
      if (Array.isArray(val)) {
        val = JSON.stringify(val);
      }
      meta += prop + ": " + val + "\n";
    }
    meta += "---\n";
    fileContent = meta + content;
    docMeta.content = content;
    docMeta.success = true;
    return {
      fileContent: fileContent,
      docMeta: docMeta
    };
  };

  checkInteger = function(num) {
    if (num === void 0 || num === null) {
      return void 0;
    }
    if (isNaN(num)) {
      return void 0;
    }
    if (!isFinite(num)) {
      return void 0;
    }
    return parseInt(num);
  };

  saveDocument = function(plugin, opts, callback) {
    var docMeta, fileContent, fullPath, ref, ref1, ref2;
    callback = callback || function() {};
    if (!opts.content || !opts.title || !opts.user) {
      callback({
        success: false,
        msg: 'missing required fields'
      });
      return;
    }
    opts.docId = checkInteger(opts.docId);
    if (!opts.docId) {
      opts.docId = (new Date()).getTime();
      opts.author = opts.author || opts.user.name;
      opts.creationdate = (new Date()).toString();
      ref = buildContent(opts, plugin), fileContent = ref.fileContent, docMeta = ref.docMeta;
      fullPath = getPath(plugin, docMeta.title);
      return writeDocument(fullPath, fileContent, docMeta, plugin, callback);
    } else {
      ref1 = mergeMetaData(plugin, opts), fullPath = ref1.fullPath, opts = ref1.opts;
      ref2 = buildContent(opts, plugin), fileContent = ref2.fileContent, docMeta = ref2.docMeta;
      return saveVersion(plugin, fullPath, opts.docId, function() {
        return writeDocument(fullPath, fileContent, docMeta, plugin, callback);
      });
    }
  };

  methods = {
    getPath: getPath,
    buildContent: buildContent,
    saveVersion: saveVersion,
    writeDocument: writeDocument,
    mergeMetaData: mergeMetaData
  };

  module.exports.saveDocument = saveDocument;

  module.exports.methods = methods;

}).call(this);
